local addonName, addonTable = ...
local GB_ChatAnswer = "|cffffcc00Guild Betrayer:|r "

--[[
	Counts the number of entries in a table.

	@return Number of elements of t
--]]
local function GetTableSize(t)
	if(t == nil) then
		return 0
	end

	local numEntries = 0

	for k in pairs(t) do
		numEntries = numEntries + 1
	end

	return numEntries
end

-- Copy frames
local CopyChat = CreateFrame("Frame", "ChatCopy", UIParent, "BackdropTemplate")
CopyChat:SetWidth(450)
CopyChat:SetHeight(300)
CopyChat:SetPoint("LEFT", UIParent, "LEFT", 3, 10)
CopyChat:SetFrameStrata("DIALOG")

CopyChat:SetBackdrop({
	bgFile = [[Interface\Buttons\WHITE8x8]],
	insets = {left = 3, right = 3, top = 4, bottom = 3}
})

CopyChat:SetBackdropColor(0, 0, 0, 0.7)
CopyChat:Hide()

--Make movable
CopyChat:EnableMouse(true)
CopyChat:SetMovable(true)
CopyChat:RegisterForDrag("LeftButton")
CopyChat:SetScript("OnDragStart", CopyChat.StartMoving)
CopyChat:SetScript("OnDragStop", CopyChat.StopMovingOrSizing)

CopyChatBox = CreateFrame("EditBox", "ChatCopyBox", CopyChat)
CopyChatBox:SetMultiLine(true)
CopyChatBox:SetAutoFocus(true)
CopyChatBox:EnableMouse(true)
CopyChatBox:SetMaxLetters(99999)
CopyChatBox:SetFont("Fonts\\ARIALN.ttf", 13, "THINOUTLINE")
CopyChatBox:SetWidth(400)
CopyChatBox:SetHeight(300)
CopyChatBox:SetScript("OnEscapePressed", function() CopyChat:Hide() end)
CopyChatBox:SetScript("OnMouseUp", function() CopyChatBox:HighlightText() end)

local Scroll = CreateFrame("ScrollFrame", "ChatCopyScroll", CopyChat, "UIPanelScrollFrameTemplate")
Scroll:SetPoint("TOPLEFT", CopyChat, "TOPLEFT", 8, -8)
Scroll:SetPoint("BOTTOMRIGHT", CopyChat, "BOTTOMRIGHT", -30, 8)
Scroll:SetScrollChild(CopyChatBox)

--[[
	Initialise saved variables on first startup.
]]--
local function SetStandardValues()
	if(GBDB == nil) then
		GBDB = {}
	end
end

-- Main Event listener.
EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("ADDON_LOADED")

EventFrame:SetScript("OnEvent", function(self, event, ...)
	if(event == "PLAYER_ENTERING_WORLD") then
		SetStandardValues()
	end
end)

--[[
	Fetches all current guild members.
	
	@return
		1st: A table with all current guild members:
			table = {
				["PlayerName1-Server"] = true,
				["PlayerName2-Server"] = true,
				...
			}
		2nd: The number of current guild members
--]]
local function GetCurrentGuildMembers()
	GuildRoster()

	local numTotalMembers = GetNumGuildMembers()
	local members = {}

	for i = 1, numTotalMembers do
		local name = GetGuildRosterInfo(i)

		members[name] = true
	end

	return members, numTotalMembers
end

--[[
	Saves every guild member to the saved variables.
]]--
local function SaveCurrentGuildMembers()
	local members, numTotalMembers = GetCurrentGuildMembers()
	GBDB = members

	print(GB_ChatAnswer .. "Scanned and saved a snapshot of the current guild members. Detected |cff00ff00" .. numTotalMembers .. "|r possible betrayers.")
end

--[[
	Scan guild members and compare them to the saved ones of the last scan.

	@return A table with the identified betrayers
]]--
local function GetBetrayers()
	if(GBDB == nil) then
		return {}
	end

	GuildRoster()

	local currentMembers, numTotalMembers = GetCurrentGuildMembers()
	local betrayers = {}

	for k in pairs(GBDB) do
		if(currentMembers[k] ~= true) then
			betrayers[k] = true
		end
	end

	return betrayers
end

--[[
	Outputs betrayer information to the chat and displays the generated HTML code.
--]]
local function OutputBetrayers()
	local betrayers = GetBetrayers()
	local betrayersHtmlList = ""

	if(GetTableSize(betrayers) == 0) then
		print(GB_ChatAnswer .. "No betrayers detected.")
	else
		for k in pairs(betrayers) do
			betrayersHtmlList = betrayersHtmlList .. "        <li>" .. k .. "</li>\n"
			print(GB_ChatAnswer .. "Betrayer detected: |cff00ff00" .. k .. "|r")
		end

		print(GB_ChatAnswer .. "Detected a total of |cff00ff00" .. GetTableSize(betrayers) .. "|r betrayers. Guilty as charged!")

		local html = addonTable.template
		html = string.gsub(html, "${date}", date("%m/%d/%y %H:%M:%S"))
		html = string.gsub(html, "${betrayers}", betrayersHtmlList)

		CopyChatBox:SetText(html)
		CopyChat:Show()
		CopyChatBox:HighlightText()
	end
end

-- Slash Commands
SLASH_GB1 = "/gb"
function SlashCmdList.GB(msg, editbox)
	-- No arguments, show the help.
	if(msg == "") then
		print(GB_ChatAnswer .. "Usage: |cff00ff00/gb scan|r or |cff00ff00/gb diff|r.")
	elseif(msg == "scan") then
		SaveCurrentGuildMembers()
	elseif(msg == "diff") then
		OutputBetrayers()
	end
end